use bevy_app::{App, AppExit};
use config::WindowConfig;
use raw_handle::{DisplayHandleWrapper, EventLoopProxyWrapper};
use state::WindowState;
use winit::event_loop::EventLoop;

pub mod config;
pub mod raw_handle;
pub mod state;

#[derive(Debug, thiserror::Error)]
pub enum WindowError {
    #[error("Failed to create event loop")]
    FailedToCreateEventLoop,
}

pub struct Window;

impl Window {
    pub fn new(app: &mut App, window_config: WindowConfig) -> Result<(), WindowError> {
        let world = app.world_mut();
        world.insert_resource(window_config);

        let mut event_loop_builder = EventLoop::with_user_event();
        let event_loop = event_loop_builder
            .build()
            .map_err(|_| WindowError::FailedToCreateEventLoop)?;

        world.insert_resource(DisplayHandleWrapper(event_loop.owned_display_handle()));

        app.set_runner(Box::new(move |app| runner(app, event_loop)));

        Ok(())
    }
}

fn runner(mut app: App, event_loop: EventLoop<()>) -> AppExit {
    app.world_mut()
        .insert_resource(EventLoopProxyWrapper::new(event_loop.create_proxy()));

    let mut window_state = WindowState::new(app);

    match event_loop.run_app(&mut window_state) {
        Ok(_) => AppExit::Success,
        Err(e) => {
            log::error!("Error running window state: {e}");
            AppExit::error()
        }
    }
}
